__includes ["../../IODA_2_2.nls"]
extensions [ioda]
 
globals [hunted-sheep sheep-births season growth-rate]
 
breed [wolves wolf]
breed [sheep a-sheep]

wolves-own  [last-dinner]
sheep-own   [stomach age female?]
patches-own [food]
  
to setup
  clear-all
  set-default-shape wolves "wolf"
  set-default-shape sheep "sheep 2"
  init-agents
  ioda:load-interactions "interactions2.txt"
  ioda:load-matrices "matrix2.txt" " \t,"
  ioda:setup
  reset-ticks
end

to go
  set season (round ticks / season-duration) mod 4
  set growth-rate item season [ 0.1 0.7 0.5 0.2]
  set hunted-sheep 0
  set sheep-births 0
  ioda:go
  tick 
end

to init-agents
  ask patches 
    [ set food random max-food
      recolor-patch ]
  create-sheep nb-sheep [ init-sheep ]
  create-wolves nb-wolves [ init-wolf ]
end

to recolor-patch
  set pcolor scale-color (item season [gray lime green orange]) food 0 (max-food * 1.2)
end

to init-sheep
  setxy random-xcor random-ycor
  set color white
  set size 1.5
  set stomach random 2 * sheep-hunger
  set age adult-sheep
  set female? random 2 = 0
end

to init-wolf
  set last-dinner (- random wolves-hunger)
  setxy random-xcor random-ycor
  set color brown
  set size 2
end

to wolves::filter-neighbors
  ioda:filter-neighbors-in-radius 5
end

to sheep::filter-neighbors
  ioda:filter-neighbors-in-radius 2
end

to-report wolves::hungry?
  report ticks - last-dinner >= wolves-hunger
end

to-report sheep::healthy?
  report true
end

to-report sheep::not-hungry?
  report not sheep::hungry?
end

to-report wolves::adult?
  report true
end

to-report sheep::alone?
  report not any? other sheep in-radius 1
end

to-report sheep::hungry?
  report stomach <= sheep-hunger
end

to-report patches::healthy?
  report food >= 0.2 * max-food
end

to-report sheep::adult?
  report age >= adult-sheep
end

to-report patches::not-grown?
  report food < max-food
end

to wolves::digest
  set last-dinner ticks
  set hunted-sheep hunted-sheep + 1
end

to sheep::die
  ioda:die
end

to wolves::chase
  face ioda:my-target  fd 1
end

to wolves::wiggle
  left random 30 right random 30 fd 1
end

to sheep::digest
  set stomach stomach + [food] of ioda:my-target
end

to patches::die
  set food 0
end

to sheep::reproduce
  set sheep-births sheep-births + 1
  let p one-of neighbors
  hatch-sheep 1 [ ioda:init-agent set age 0 set heading random 360 fd 0.8 set color white set size 1 
    set stomach 10 set female? (random 2 = 0) create-link-with myself [ tie ]]
end

to sheep::decrease-health
  ifelse female? [ set stomach stomach - 8]
    [ set stomach stomach - 4]
end

to sheep::wiggle
  left random 60 right random 60 fd 1
end

to patches::increase-size
  set food food + growth-rate
  recolor-patch
end

to-report sheep::female?
  report female?
end

to-report sheep::male?
  report not sheep::female?
end

to-report sheep::has-child?
  report any? link-neighbors
end

to-report sheep::few-children?
  report count my-links < 2
end

to-report sheep::not-grown?
  report not sheep::adult?
end

to sheep::increase-size
  set age age + 1
  if (age >= adult-sheep) 
    [ set size 1.5
      ask my-links [ untie die ]]
end

to-report sheep::starving?
  report stomach <= 0
end

to sheep::nurse
  let t ioda:my-target
  let q 1 + round(stomach / 2)
  set stomach stomach - q
  ask t [set stomach stomach + q]
end

to sheep::consume-energy
  set stomach stomach - 1
end

to sheep::move-away
  face ioda:my-target
  right 180 fd 1
end





@#$#@#$#@
GRAPHICS-WINDOW
590
10
1020
461
10
10
20.0
1
10
1
1
1
0
1
1
1
-10
10
-10
10
0
0
1
ticks
30.0

BUTTON
369
10
435
43
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
444
10
507
43
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
11
144
582
460
Population
time
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"newborn sheep" 1.0 1 -11221820 true "" "plot count sheep with [sheep::not-grown?]"
"adult sheep" 1.0 0 -13345367 true "" "plot count sheep with [sheep::adult?]"
"adult females" 1.0 0 -13791810 true "" "plot count sheep with [sheep::adult? and female?]"
"grass (/10)" 1.0 0 -10899396 true "" "plot sum [food] of patches / 10"

SLIDER
9
10
181
43
nb-sheep
nb-sheep
0
100
100
1
1
NIL
HORIZONTAL

SLIDER
10
52
182
85
nb-wolves
nb-wolves
0
20
10
1
1
NIL
HORIZONTAL

SLIDER
188
10
360
43
max-food
max-food
0
20
10
1
1
NIL
HORIZONTAL

SLIDER
9
96
181
129
adult-sheep
adult-sheep
0
100
40
1
1
NIL
HORIZONTAL

SLIDER
188
52
360
85
wolves-hunger
wolves-hunger
0
200
80
1
1
NIL
HORIZONTAL

SLIDER
188
96
360
129
sheep-hunger
sheep-hunger
0
100
40
1
1
NIL
HORIZONTAL

SLIDER
369
55
541
88
season-duration
season-duration
10
200
50
10
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

This model presents a simple ecosystem with three agent families: grass (patches), sheep (a NeLogo breed) and wolves (another breed). It is the implementation of the enhancement proposed in the section 4 of the [IODA NetLogo Tutorial](file:../../doc/IODA-NetLogo-Tutorial.html).

## HOW IT WORKS

Several model extensions are proposed in the Tutorial, e.g.:  

  * make sheep die by starvation  
  * make sheep flee to escape wolves   
  * introduce sexual differentiation to enhance the realism of the _`Mate`_ interaction
  * make sheep grow from newborn to adult  
  * force newborn to feed from their mother  
  * limit female mating, depending on the number of newborns  
  * introduce seasonal variations in the food growth rate  

This model is an implementation of those extensions.

## IODA NETLOGO FEATURES

This model introduces several IODA NetLogo features:   

  1. the use of patches as "classical" agents: in IODA NetLogo, patches are treated exactly as turtles or other breeds, since each entity of the model should be represented by an agent  
  2. the use of the Update Matrix, which allows to define how agents should update their state (i.e. with what interactions) before performing or undergoing interactions from the Interaction Matrix  
  3. the difference between two classes of interactions: **exclusive interactions** vs. **parallel interactions**. Parallel interactions do not affect the ability to participate in other interactions, while exclusive interactions prevent agents from participating in other exclusive interactions  
  4. notwithstanding the ability to use the same interaction (such as _`Eat`_) for different breeds (wolves and sheep), the definition of concrete primitives provides a powerful way to introduce differences in the modalities of performing or undergoing an interaction (see e.g. how the _hungry?_ abstract primitive, used in the _`Eat`_ interaction, is implemented in the concrete `wolves::hungry?` and `sheep::hungry?` reporters).

## RELATED MODELS

This model is close to many predator/prey simulations, such as the "Wolf Sheep Predation" model in the NetLogo library, though it is not aimed at demonstrating equilibria but rather IODA features. It is an extension of the file [`"model1.nlogo"`](file:model1.nlogo).

## HOW TO CITE

  * The **IODA methodology and simulation algorithms** (i.e. what is actually in use in this NetLogo extension):  
Y. KUBERA, P. MATHIEU and S. PICAULT (2011), "IODA: an interaction-oriented approach for multi-agent based simulations", in: _Journal of Autonomous Agents and Multi-Agent Systems (JAAMAS)_, vol. 23 (3), p. 303-343, Springer DOI: 10.1007/s10458-010-9164-z.  
  * The **key ideas** of the IODA methodology:  
P. MATHIEU and S. PICAULT (2005), "Towards an interaction-based design of behaviors", in: M.-P. Gleizes (ed.), _Proceedings of the The Third European Workshop on Multi-Agent Systems (EUMAS'2005)_.  
  * Do not forget to cite also **NetLogo** itself when you refer to the IODA NetLogo extension:  
U. WILENSKY (1999), NetLogo. http://ccl.northwestern.edu/netlogo Center for Connected Learning and Computer-Based Modeling, Northwestern University. Evanston, IL.

## COPYRIGHT NOTICE

All contents &copy; 2008-2013 Sébastien PICAULT and Philippe MATHIEU  
Laboratoire d'Informatique Fondamentale de Lille (LIFL), UMR CNRS 8021   
University Lille 1 --- Cité Scientifique, F-59655 Villeneuve d'Ascq Cedex, FRANCE.  
Web Site: http://www.lifl.fr/SMAC/projects/ioda

The IODA NetLogo extension is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

IODA NetLogo extension is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with the IODA NetLogo extension. If not, see http://www.gnu.org/licenses.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

sheep 2
false
0
Polygon -7500403 true true 209 183 194 198 179 198 164 183 164 174 149 183 89 183 74 168 59 198 44 198 29 185 43 151 28 121 44 91 59 80 89 80 164 95 194 80 254 65 269 80 284 125 269 140 239 125 224 153 209 168
Rectangle -7500403 true true 180 195 195 225
Rectangle -7500403 true true 45 195 60 225
Rectangle -16777216 true false 180 225 195 240
Rectangle -16777216 true false 45 225 60 240
Polygon -7500403 true true 245 60 250 72 240 78 225 63 230 51
Polygon -7500403 true true 25 72 40 80 42 98 22 91
Line -16777216 false 270 137 251 122
Line -16777216 false 266 90 254 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -7500403 true true 75 225 97 249 112 252 122 252 114 242 102 241 89 224 94 181 64 113 46 119 31 150 32 164 61 204 57 242 85 266 91 271 101 271 96 257 89 257 70 242
Polygon -7500403 true true 216 73 219 56 229 42 237 66 226 71
Polygon -7500403 true true 181 106 213 69 226 62 257 70 260 89 285 110 272 124 234 116 218 134 209 150 204 163 192 178 169 185 154 189 129 189 89 180 69 166 63 113 124 110 160 111 170 104
Polygon -6459832 true true 252 143 242 141
Polygon -6459832 true true 254 136 232 137
Line -16777216 false 75 224 89 179
Line -16777216 false 80 159 89 179
Polygon -6459832 true true 262 138 234 149
Polygon -7500403 true true 50 121 36 119 24 123 14 128 6 143 8 165 8 181 7 197 4 233 23 201 28 184 30 169 28 153 48 145
Polygon -7500403 true true 171 181 178 263 187 277 197 273 202 267 187 260 186 236 194 167
Polygon -7500403 true true 187 163 195 240 214 260 222 256 222 248 212 245 205 230 205 155
Polygon -7500403 true true 223 75 226 58 245 44 244 68 233 73
Line -16777216 false 89 181 112 185
Line -16777216 false 31 150 47 118
Polygon -16777216 true false 235 90 250 91 255 99 248 98 244 92
Line -16777216 false 236 112 246 119
Polygon -16777216 true false 278 119 282 116 274 113
Line -16777216 false 189 201 203 161
Line -16777216 false 90 262 94 272
Line -16777216 false 110 246 119 252
Line -16777216 false 190 266 194 274
Line -16777216 false 218 251 219 257
Polygon -16777216 true false 230 67 228 54 222 62 224 72
Line -16777216 false 246 67 234 64
Line -16777216 false 229 45 235 68
Line -16777216 false 30 150 30 165

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.4
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
